/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2007
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.ComponentModel;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using DNNContentBuilder;

namespace DNNContentBuilder.UI.WebControls
{
	/// <summary>
	/// A web control that encapsulates all the rendering and processing logic of a DataGrid, DataList or
	/// Repeater paging system.
	/// </summary>
	/// <example>
	/// 
	/// <p><b>DataGrid Integration</b></p>
	/// 
	/// <p>
	/// It is assumed that you have previously added the code to bind the data to your DataGrid in the
	/// <code>Page_Load</code> or using the appropriate designer.
	/// </p>
	/// <code>
	/// <![CDATA[
	/// <asp:DataGrid id="DataGrid1" Runat="server" Width="100%" />
	/// <tax:PagerBuilder id="PagerBuilder1" 
	///					  Width="100%" PageSize="5" 
	///					  Runat="server" 
	///					  ControlToPage="DataGrid1" />
	/// ]]!>
	/// </code>
	/// 
	/// <p><b>DataList or Repeater Integration</b></p>
	/// 
	/// <code>
	/// <![CDATA[
	/// <asp:DataList id="DataList1" Runat="server" Width="100%">
	///		<ItemTemplate>
	///			<b><% DataBinder.Eval(Container.DataItem, "codigo")    %></b>
	///			<b><% DataBinder.Eval(Container.DataItem, "descricao") %></b>
	///		</ItemTemplate>
	/// </asp:DataList>
	/// <tax:PagerBuilder id="PagerBuilder1" 
	///					  Width="100%" PageSize="5" 
	///					  Runat="server" 
	///					  ControlToPage="DataList1" />
	/// ]]!>
	/// 
	/// </code>
	/// </example>
	[ToolboxData("<{0}:PagerBuilder Runat=server></{0}:PagerBuilder>"), Designer(typeof(PagerBuilderDesigner))]
	public class PagerBuilder : System.Web.UI.Control, System.Web.UI.INamingContainer
	{
		#region Class Variables
		private int				_borderWidth;
		private int				_cellPadding;
		private int				_cellSpacing;
		private string			_cssClass;
		private bool			_debug;
		private string			_firstText;
		private string			_infoTemplate;
		private string			_lastText;
		private string			_navSeparator;
		private string			_nextText;
		private string			_panelCssClass;
		private string			_previousText;
		private Style			_style;
		private bool			_useQuery;
		private Unit			_height;
		private Unit			_width;
		private bool			_causeValidation;
		private bool			_firstLastDisabled;
		private bool			_infoPanelDisabled;
		private bool			_pageGroupDisabled;
		private HorizontalAlign	_infoPanelHorizontalAlign;
		private VerticalAlign	_infoPanelVerticalAlign;
		private HorizontalAlign	_navPanelHorizontalAlign;
		private VerticalAlign	_navPanelVerticalAlign;

		public const string resource = "~/DesktopModules/DNNContentBuilder/App_LocalResources/PagerBuilder";
		#endregion

		#region Class Events
		public event EventHandler IndexChanged;
		#endregion

		#region Constructors
		public PagerBuilder() : base()
		{
			_debug						= false;
			_useQuery					= false;
			_infoPanelDisabled			= false;
			_firstLastDisabled			= false;
			_pageGroupDisabled			= false;
			_causeValidation			= false;
			CurrentIndex				= 0;
			PageSize					= 1;
			RecordCount					= 0;
			_cellPadding				= 2;
			_cellSpacing				= 1;
			_borderWidth				= 0;
			_cssClass					= string.Empty;
			_panelCssClass				= string.Empty;
			_navPanelHorizontalAlign	= HorizontalAlign.Right;;
			_previousText				= "<b>" + Commons.GetLocalizedString("label_Previous", resource) + "</b>";
			_firstText					= "<b>" + Commons.GetLocalizedString("label_First", resource) + "</b>";
			_lastText					= "<b>" + Commons.GetLocalizedString("label_Last", resource) + "</b>";
			_nextText					= "<b>" + Commons.GetLocalizedString("label_Next", resource) + "</b>";
			_navSeparator				= "&nbsp;&nbsp;&nbsp;";
			_infoTemplate				=  Commons.GetLocalizedString("label_Total", resource) + ": <b>$RECORDCOUNT$</b> - "  + Commons.GetLocalizedString("label_Page", resource) + ": <b>$CURRENTPAGE$</b> " + Commons.GetLocalizedString("label_Of", resource) + " <b>$PAGECOUNT$</b>";
		}
		#endregion

		#region Properties
		/// <summary>
		/// Specify whether the link button cause the validaton process
		/// to start when clicked.
		/// </summary>
		public bool CausesValidation
		{
			get
			{
				return _causeValidation;
			}
			set
			{
				_causeValidation = value;
			}
		}

		/// <summary>
		/// Gets or sets the cell padding value of the table that contains 
		/// the pager panels.
		/// </summary>
		public int CellPadding
		{
			get
			{
				return _cellPadding;
			}
			set
			{
				_cellPadding = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the cell spacing value of the table that contain 
		/// the page panels.
		/// </summary>
		public int CellSpacing
		{
			get
			{
				return _cellSpacing;
			}
			set
			{
				_cellSpacing = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the control to page.
		/// </summary>
		public string ControlToPage
		{
			get
			{
				object savedState = ViewState["ControlToPage"];

				if (savedState != null)
				{
					return (string)savedState;
				}

				return string.Empty;
			}
			set
			{
				ViewState["ControlToPage"] = value;		
			}
		}

		/// <summary>
		/// Gets the current page group.
		/// </summary>
		public int CurrentGroup
		{
			get
			{
				if (CurrentPage % 10 == 0)
				{
					return ((CurrentPage - 9) / 10) + 1;
				}

				return (CurrentPage / 10) + 1;
			}
		}

		/// <summary>
		/// Gets or sets the current index.
		/// </summary>
		public int CurrentIndex
		{
			get
			{
				this.EnsureChildControls();
				return (Int32)ViewState["_currentIndex"];
			}
			set
			{
				ViewState["_currentIndex"] = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the current page index.
		/// </summary>
		public int CurrentPage
		{
			get
			{
				if (CurrentIndex > 0)
				{
					return CurrentIndex / PageSize + 1;
				}

				return 1;
			}
			set
			{
				CurrentIndex = value + 1 * PageSize;
			}
		}

		/// <summary>
		/// Specify whether the first and last page links are
		/// displayed or not.
		/// </summary>
		public bool FirstLastDisabled
		{
			get
			{
				return _firstLastDisabled;
			}
			set
			{
				_firstLastDisabled = value;
			}
		}

		/// <summary>
		/// Gets or sets the text to display for the first page
		/// link.
		/// </summary>
		public string FirstText
		{
			get
			{
				return _firstText;
			}
			set
			{
				_firstText = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the height of the pager.
		/// </summary>
		public Unit Height
		{
			get
			{
				return _height;
			}
			set
			{
				_height = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the horizontal alignment of the pager.
		/// </summary>
		public HorizontalAlign HorizontalAlign
		{
			get
			{
				if (ViewState["_horizontalAlign"] == null)
				{
					ViewState["_horizontalAlign"] = HorizontalAlign.NotSet;
				}

				return (HorizontalAlign)ViewState["_horizontalAlign"];
			}
			set
			{
				ViewState["_horizontalAlign"] = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Specify whether the information panel is displayed
		/// or not.
		/// </summary>
		public bool InfoPanelDisabled
		{
			get
			{
				return _infoPanelDisabled;
			}
			set
			{
				_infoPanelDisabled = value;
			}
		}

		/// <summary>
		/// Gets or sets the horizontal alignment of the information
		/// panel content.
		/// </summary>
		public HorizontalAlign InfoPanelHorizontalAlign
		{
			get
			{
				return _infoPanelHorizontalAlign;
			}
			set
			{
				_infoPanelHorizontalAlign = value;
			}
		}

		/// <summary>
		/// Gets or sets the vertical alignment of the information
		/// panel content.
		/// </summary>
		public VerticalAlign InfoPanelVerticalAlign
		{
			get
			{
				return _infoPanelVerticalAlign;
			}
			set
			{
				_infoPanelVerticalAlign = value;
			}
		}

		/// <summary>
		/// Gets or sets the information template of the pager.
		/// </summary>
		/// <remarks>
		/// <p>
		/// The default value of this property is 
		/// <code><![CDATA[
		/// Total: <b>$RECORDCOUNT</b> - Pgina <b>$CURRENTPAGE</b> de <b>$PAGECOUNT</b>
		/// ]]></code>.
		/// </p>
		/// 
		/// <p>
		/// The value of this property consists of a string containing both literal and tag
		/// content. To display the record count, you must include $RECORDCOUNT$.
		/// </p>
		/// 
		/// <p>The available tags are:</p>
		/// 
		/// <p>
		/// <code>$RECORDCOUNT$ </code>: display the total number of records contained in the data source.
		/// <code>$CURRENTPAGE$ </code>: display the current selected page in the data source.
		/// <code>$PAGECOUNT$   </code>: display the total number of pages contained in the data source.
		/// <code>$CURRENTINDEX$</code>: display the current index.
		/// </p>
		/// </remarks>
		public string InfoTemplate
		{
			get
			{
				return _infoTemplate;
			}
			set
			{
				_infoTemplate = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the text to display for the last page link.
		/// </summary>
		public string LastText
		{
			get
			{
				return _lastText;
			}
			set
			{
				_lastText = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the horizontal alignment of the navigation 
		/// panel content.
		/// </summary>
		public HorizontalAlign NavPanelHorizontalAlign
		{
			get
			{
				return _navPanelHorizontalAlign;
			}
			set
			{
				_navPanelHorizontalAlign  = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the vertical alignment of the navigation
		/// panel content.
		/// </summary>
		public VerticalAlign NavPanelVerticalAlign
		{
			get
			{
				return _navPanelVerticalAlign;
			}
			set
			{
				_navPanelVerticalAlign	  = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the text to use as a separator between
		/// navigation elements.
		/// </summary>
		public string NavSeparator
		{
			get
			{
				return _navSeparator;
			}
			set
			{
				_navSeparator = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the text to display for the next page link.
		/// </summary>
		public string NextText
		{
			get
			{
				return _nextText;
			}
			set
			{
				_nextText = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets the total number of pages in the paged list;
		/// </summary>
		public int PageCount
		{
			get
			{
				if (RecordCount % PageSize > 0)
				{
					return RecordCount / PageSize + 1;
				}

				return RecordCount / PageSize;
			}
		}

		/// <summary>
		/// Specify whether the page group navigation is displayed
		/// or not.
		/// </summary>
		public bool PageGroupDisabled
		{
			get
			{
				return _pageGroupDisabled;
			}
			set
			{
				_pageGroupDisabled = value;
			}
		}

		/// <summary>
		/// Gets or sets the style of the pager.
		/// </summary>
		/// <remarks>
		/// Best used for applying styles for both the information panel and the
		/// navigation panel.
		/// </remarks>
		public Style PagerStyle
		{
			get
			{
				if (_style == null)
				{
					_style = new Style();
				}

				return _style;
			}
			set
			{
				_style = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the number of items to display per page.
		/// </summary>
		/// <remarks>
		/// Only values greater than zero are allowed. Settings this property
		/// to 0 or -1 will apply the default value of 1.
		/// </remarks>
		public int PageSize
		{
			get
			{
				this.EnsureChildControls();
				return (Int32)ViewState["_pageSize"];
			}
			set
			{
				ViewState["_pageSize"]	  = (value < 1) ? 1 : value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the Cascading Style Sheet name of the panel
		/// where the status is displayed.
		/// </summary>
		/// <remarks>
		/// This property will affect the table line (TR) of the control
		/// that displays both the info panel and navigation panel. To
		/// apply borders or other styles, use the <see cref="PagerStyle"/>
		/// property.
		/// </remarks>
		public string PanelCssClass
		{
			get
			{
				return _panelCssClass;
			}
			set
			{
				_panelCssClass = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the text to display for the previous page link.
		/// </summary>
		public string PreviousText
		{
			get
			{
				return _previousText;
			}
			set
			{
				_previousText = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the total number of records contained in the 
		/// paged list.
		/// </summary>
		public int RecordCount
		{
			get
			{
				this.EnsureChildControls();
				return (Int32)ViewState["_recordCount"];
			}
			set
			{
				ViewState["_recordCount"] = value;
				this.ChildControlsCreated = false;
			}
		}

		/// <summary>
		/// Gets or sets the width of the pager.
		/// </summary>
		public Unit Width
		{
			get
			{
				return _width;
			}
			set
			{
				_width = value;
				this.ChildControlsCreated = false;
			}
		}
		#endregion

		#region Implementation
		internal string BuildInfo(bool designTime) 
		{
			string result;

			result = _infoTemplate;

			if (!(designTime)) 
			{
				result = result.Replace("$CURRENTINDEX$", CurrentIndex.ToString("##"));
				result = result.Replace("$RECORDCOUNT$", RecordCount.ToString("##"));
				result = result.Replace("$CURRENTPAGE$", CurrentPage.ToString("##"));
				result = result.Replace("$PAGECOUNT$",	PageCount.ToString("##"));
			}
			
			return result;
		}

		/// <summary>
		/// Creates the child controls.
		/// </summary>
		protected override void CreateChildControls() 
		{
			LinkButton previousPageButton;
			LinkButton firstPageButton;
			LinkButton previousGroupButton;
			LinkButton gotoPageButton;
			LinkButton nextGroupButton;
			LinkButton lastPageButton;
			LinkButton nextPageButton;

			int currentGroup;
			int lastPageInGroup;
			int currentPageInGroup;
			
			Page.Trace.Write("In CreateChildControls");
			this.PrintStatus();

			Table table			= new Table();
			TableRow panel		= new TableRow();
			TableCell infoPanel	= new TableCell();
			TableCell navPanel	= new TableCell();

			panel.Cells.Add(infoPanel);
			panel.Cells.Add(navPanel);
			
			table.Rows.Add(panel);
			table.ApplyStyle(this.PagerStyle);
			
			table.CssClass			= _cssClass;
			table.BorderWidth		= _borderWidth;
			table.CellPadding		= _cellPadding;
			table.CellSpacing		= _cellSpacing;
			table.Width				= this.Width;
			table.Height			= this.Height;
			table.HorizontalAlign	= this.HorizontalAlign;
			
			panel.CssClass = _panelCssClass;
			
			infoPanel.ApplyStyle(this.PagerStyle);

			infoPanel.Visible         = (this.InfoPanelDisabled == false);
			infoPanel.VerticalAlign   = _infoPanelVerticalAlign;
			infoPanel.HorizontalAlign = _infoPanelHorizontalAlign;
			
			navPanel.ApplyStyle(this.PagerStyle);
			navPanel.HorizontalAlign = _navPanelHorizontalAlign;
			navPanel.VerticalAlign	 = _navPanelVerticalAlign;
			
			Label status = new Label();
			status.Text  = this.BuildInfo(false);
			
			infoPanel.Controls.Add(status);
			
			previousPageButton = new LinkButton();

			if (CurrentIndex == 0)
			{
				previousPageButton.Enabled = false;
			}
			
			previousPageButton.CausesValidation = this.CausesValidation;
			previousPageButton.Click += new EventHandler(this.OnPreviousPage_Click);
			previousPageButton.Text   = _previousText;

			navPanel.Controls.Add(previousPageButton);
			navPanel.Controls.Add(new LiteralControl(_navSeparator));
			
			if (!FirstLastDisabled) 
			{
				firstPageButton = new LinkButton();
				
				if (CurrentIndex == 0)
				{
					firstPageButton.Enabled = false;
				}
				
				firstPageButton.CausesValidation = this.CausesValidation;
				firstPageButton.Click += new EventHandler(this.OnFirstPage_Click);
				firstPageButton.Text = _firstText;
				navPanel.Controls.Add(firstPageButton);
				navPanel.Controls.Add(new LiteralControl(_navSeparator));
			}

			if (!PageGroupDisabled) 
			{
				currentGroup    = CurrentPage % 10 == 0 ? CurrentPage - 9 : CurrentPage - CurrentPage % 10 + 1;
				lastPageInGroup = currentGroup + 9;
				lastPageInGroup = lastPageInGroup > this.PageCount ? PageCount : lastPageInGroup;

				previousGroupButton = new LinkButton();
				
				if (CurrentGroup == 1)
				{
					previousGroupButton.Enabled = false;
				}
				
				previousGroupButton.CausesValidation = this.CausesValidation;
				previousGroupButton.Click += new EventHandler(this.OnPreviousGroup_Click);
				previousGroupButton.Text   = "&lt;";
				
				navPanel.Controls.Add(previousGroupButton);
				navPanel.Controls.Add(new LiteralControl("["));
				
				currentPageInGroup = currentGroup;
				
				while (currentPageInGroup <= lastPageInGroup) 
				{
					if (currentPageInGroup != CurrentPage) 
					{
						gotoPageButton = new LinkButton();
						
						gotoPageButton.CausesValidation = this.CausesValidation;
						gotoPageButton.Click += new EventHandler(this.OnGotoPage_Click);
						gotoPageButton.Text   = currentPageInGroup.ToString();
						
						navPanel.Controls.Add(gotoPageButton);
					}
					else
					{
						navPanel.Controls.Add(new LiteralControl("<b>" + currentPageInGroup.ToString() + "</b>"));
					}
					
					if (currentPageInGroup != lastPageInGroup)
					{
						navPanel.Controls.Add(new LiteralControl("&nbsp;"));
					}

					currentPageInGroup++;
				}

				navPanel.Controls.Add(new LiteralControl("]"));
				
				nextGroupButton = new LinkButton();
				
				if (CurrentGroup == (PageCount + 9) / 10)
				{
					nextGroupButton.Enabled = false;
				}
				
				nextGroupButton.CausesValidation = this.CausesValidation;
				nextGroupButton.Text = "&gt;";
				nextGroupButton.Click += new EventHandler(this.OnNextGroup_Click);
				navPanel.Controls.Add(nextGroupButton);
			}

			if (!FirstLastDisabled) 
			{
				navPanel.Controls.Add(new LiteralControl(_navSeparator));
				lastPageButton = new LinkButton();

				if (CurrentIndex >= RecordCount - PageSize)
				{
					lastPageButton.Enabled = false;
				}
				
				lastPageButton.CausesValidation = this.CausesValidation;
				lastPageButton.Click += new EventHandler(this.OnLastPage_Click);
				lastPageButton.Text = _lastText;
				
				navPanel.Controls.Add(lastPageButton);
				navPanel.Controls.Add(new LiteralControl(_navSeparator));
			}
			
			nextPageButton = new LinkButton();
			
			if (CurrentIndex >= RecordCount - PageSize)
			{
				nextPageButton.Enabled = false;
			}
			
			nextPageButton.CausesValidation = this.CausesValidation;
			nextPageButton.Click += new EventHandler(this.OnNextPage_Click);
			nextPageButton.Text = _nextText;

			navPanel.Controls.Add(nextPageButton);
			
			this.Controls.Add(table);
			this.PrintStatus();
		}

		private void DebugTrace(string line) 
		{
			if (_debug)
			{
				Page.Trace.Write("PagerBuilder", line);
			}
		}

		private int GetRecordCount(object dataSource) 
		{
			if ((dataSource == null) || !(dataSource is DataTable))
			{
				return 0;
			}

			return ((DataTable)dataSource).Rows.Count;
		}
	
		/// <summary>
		/// Gets the record index.
		/// </summary>
		/// <returns>The current record index.</returns>
		/// <remarks>Please do not use this property.</remarks>
		public int GetRecordIndex() 
		{
			return CurrentIndex * PageSize;
		}

		private void InitControlToPage() 
		{
			Control controlToPage;

			Page.Trace.Write("In InitControlToPage");
			controlToPage = Page.FindControl(ControlToPage);
			
			if (controlToPage != null && controlToPage is DataGrid) 
			{
				DataGrid datagrid = controlToPage as DataGrid;

				datagrid.AllowPaging		= true;
				datagrid.AllowCustomPaging	= false;
				datagrid.PagerStyle.Visible	= false;
				datagrid.PageSize			= this.PageSize;
				datagrid.CurrentPageIndex	= CurrentPage - 1;
				
				datagrid.DataBind();

				if (RecordCount == 0) 
				{
					RecordCount = this.GetRecordCount(datagrid.DataSource);
				}
			}
		}

		private void OnFirstPage_Click(object sender, EventArgs e) 
		{
			DebugTrace("In OnFirstPage_Click");
			CurrentIndex = 0;
			OnIndexChanged(EventArgs.Empty);
		}

		private void OnGotoPage_Click(object sender, EventArgs e) 
		{
			DebugTrace("In OnGoto_Click");
			
			int page = Convert.ToInt32(((LinkButton)sender).Text);
			CurrentIndex = page * PageSize - PageSize;

			OnIndexChanged(EventArgs.Empty);
		}

		/// <summary>
		/// Call all <see cref="IndexChanged"/> event handlers, if any.
		/// </summary>
		/// <param name="e">The event arguments.</param>
		protected virtual void OnIndexChanged(EventArgs e) 
		{
			Page.Trace.Write("In OnIndexChanged");
			InitControlToPage();
			
			if (this.IndexChanged != null)
			{
				IndexChanged.Invoke(this, e);
			}
		}

		/// <summary>
		/// Initializes the pager.
		/// </summary>
		/// <param name="e">The event arguments.</param>
		protected override void OnInit(EventArgs e) 
		{
			Page.Trace.Write("In OnInit");
			
			if (!(Page.IsPostBack) && ControlToPage != null && ControlToPage.Length > 0)
			{
				InitControlToPage();
			}
		}

		private void OnLastPage_Click(object sender, EventArgs e) 
		{
			int offset;

			DebugTrace("In OnLast_Click");

			offset		 = RecordCount % PageSize;
			CurrentIndex = (offset > 0) ? RecordCount - offset : RecordCount - PageSize;
			
			OnIndexChanged(EventArgs.Empty);
		}

		private void OnNextGroup_Click(object sender, EventArgs e) 
		{
			DebugTrace("In OnNextGroup_Click");
			CurrentIndex = PageSize * 10 * CurrentGroup;
			
			OnIndexChanged(EventArgs.Empty);
		}

		private void OnNextPage_Click(object sender, EventArgs e) 
		{
			DebugTrace("In OnNext_Click");
			
			if (CurrentIndex + 1 < RecordCount)
			{
				CurrentIndex = CurrentIndex + PageSize;
			}
			
			OnIndexChanged(EventArgs.Empty);
		}

		/// <summary>
		/// Overrides <see cref="System.Web.UI.Control.OnPreRender"/>.
		/// </summary>
		/// <param name="e">The event arguments.</param>
		protected override void OnPreRender(EventArgs e) 
		{
			Page.Trace.Write("In OnPreRender");
			
			if (ControlToPage != null && ControlToPage.Length > 0)
			{
				InitControlToPage();
			}

			if (_useQuery) 
			{
				this.ChildControlsCreated = false;
				OnIndexChanged(EventArgs.Empty);
			}
		}

		private void OnPreviousGroup_Click(object sender, EventArgs e) 
		{
			CurrentIndex = PageSize * 10 * (CurrentGroup - 2);
			DebugTrace("In OnPreviousGroup_Click");
			
			OnIndexChanged(EventArgs.Empty);
		}

		private void OnPreviousPage_Click(object sender, EventArgs e) 
		{
			DebugTrace("In OnPreviousPage_Click");
			CurrentIndex = CurrentIndex - PageSize;
			
			if (CurrentIndex < 0)
			{
				CurrentIndex = 0;
			}

			OnIndexChanged(EventArgs.Empty);
		}

		private void PrintStatus() 
		{
			DebugTrace("CurrentIndex = " + CurrentIndex.ToString());
			DebugTrace("PageSize     = " + PageSize.ToString());
			DebugTrace("RecordCount  = " + RecordCount.ToString());
			DebugTrace("CurrentGroup = " + CurrentGroup.ToString());
		}

		/// <summary>
		/// Overloaded version, gets a DataView, returns a paged DataTable
		/// </summary>
		/// <param name="dv"></param>
		/// <param name="currentIndex"></param>
		/// <param name="pageSize"></param>
		/// <returns></returns>
		public static DataTable GetDataTable(DataView dv, int currentIndex, int pageSize)
		{
			DataTable result = dv.Table.Clone();

			result.Rows.Clear();

			int lastIndex = currentIndex + pageSize;
			lastIndex = (lastIndex > dv.Count) ? dv.Count : lastIndex;

			for (int i = currentIndex; i < lastIndex; i++)
			{
				result.ImportRow(dv[i].Row);
			}

			return result;
		}

		/// <summary>
		/// Overloaded version, gets a DataTable, also returns a paged DataTable
		/// </summary>
		/// <param name="dt"></param>
		/// <param name="currentIndex"></param>
		/// <param name="pageSize"></param>
		/// <returns></returns>
		public static DataTable GetDataTable(DataTable dt, int currentIndex, int pageSize)
		{
			DataTable result = dt.Clone();

			result.Rows.Clear();

			int lastIndex = currentIndex + pageSize;
			lastIndex = (lastIndex > dt.Rows.Count) ? dt.Rows.Count : lastIndex;

			for (int i = currentIndex; i < lastIndex; i++)
			{
				result.ImportRow(dt.Rows[i]);
			}

			return result;
		}

		/// <summary>
		/// Renders the control to the specified output.
		/// </summary>
		/// <param name="output">The HTML writer to write out to.</param>
		protected override void Render(HtmlTextWriter output) 
		{
			this.EnsureChildControls();
			base.Render(output);
		}
		#endregion
	}
}
