/*
 DNN Content Builder - sourceforge.net/projects/dnncb
 Copyright (c) 2004-2007
 by Emerson Lopes (emersonlopes@gmail.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
 the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 to permit persons to whom the Software is furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all copies or substantial portions 
 of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
 TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Globalization;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

namespace DNNContentBuilder.UI.WebControls
{
	/// <summary>
	/// A version of the RadioButtonEx that is not affected by the NamingContainer, but
	/// whose GroupName property behaves like a regular HTML input type="radio".
	/// </summary>
	[ToolboxData("<{0}:RadioButtonEx runat=server />")]
	public class RadioButtonEx : System.Web.UI.WebControls.RadioButton, IPostBackDataHandler
	{
		#region Properties

		[Bindable(true)]
		[Category("Behavior")]
		[DefaultValue("")]
		public string RadioValue 
		{
			get 
			{ 
				return(string)this.ViewState["RadioValue"];
			}
			set 
			{ 
				this.ViewState["RadioValue"] = value;
			}
		}

		[Browsable(false)]
		private string ValueAttribute
		{
			get
			{
				string value = base.Attributes["value"];
				
				if (value != null)
				{
					return value;
				}
				
				if (this.ID != null)
				{
					return this.ID;
				}
				
				return this.UniqueID;
			}
		}

		[Bindable(true)]
		[Category("Behavior")]
		[DefaultValue("")]
		public override string GroupName
		{
			get
			{
				return (string)this.ViewState["GroupName"];
			}
			set
			{
				this.ViewState["GroupName"] = value;
			}
		}
		#endregion

		#region Implementation
		private void RenderLabel(HtmlTextWriter writer, string text, string clientID)
		{
			writer.AddAttribute(HtmlTextWriterAttribute.For, clientID);
			writer.RenderBeginTag(HtmlTextWriterTag.Label);
			writer.Write(text);
			writer.RenderEndTag();
		}

		/// <summary>
		/// Overrides <see cref="CheckBox.Render"/>
		/// </summary>
		/// <param name="writer"></param>
		protected override void Render(System.Web.UI.HtmlTextWriter writer)
		{
			if (this.Page != null)
			{
				this.Page.VerifyRenderingInServerForm(this);
			}
			
			bool isChecked = false;
			
			if (base.ControlStyleCreated)
			{
				Style controlStyle = base.ControlStyle;
				
				if (controlStyle != null)
				{
					controlStyle.AddAttributesToRender(writer, this);
					isChecked = true;
				}
			}
			
			if (!this.Enabled)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Disabled, "disabled");
				isChecked = true;
			}
			
			if (this.ToolTip.Length > 0)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Title, this.ToolTip);
				isChecked = true;
			}
			
			string onClick = null;
			
			if (base.Attributes.Count > 0)
			{
				string value = Attributes["value"];
				
				if (value != null)
				{
					Attributes.Remove("value");
				}
				
				onClick = Attributes["onclick"];
				
				if (onClick != null)
				{
					Attributes.Remove("onclick");
				}
				
				if (Attributes.Count != 0 )
				{
					Attributes.AddAttributes(writer);
					isChecked = true;
				}
				
				if (value != null)
				{
					Attributes["value"] = value;
				}
			}
			
			if (isChecked)
			{
				writer.RenderBeginTag(HtmlTextWriterTag.Span);
			}
			
			if (this.Text.Length != 0)
			{
				if (this.TextAlign == TextAlign.Left)
				{
					this.RenderLabel(writer, this.Text, this.ClientID);
					this.RenderInputTag(writer, this.ClientID, onClick);
				}
				else
				{
					this.RenderInputTag(writer, this.ClientID, onClick);
					this.RenderLabel(writer, this.Text, this.ClientID);
				}
			}
			else
			{
				this.RenderInputTag(writer, this.ClientID, onClick);
			}
			
			if (isChecked)
			{
				writer.RenderEndTag();
			}
		}

		private void RenderInputTag(HtmlTextWriter writer, string clientID, string onClick)
		{
			writer.AddAttribute(HtmlTextWriterAttribute.Id, clientID);
			writer.AddAttribute(HtmlTextWriterAttribute.Type, "radio");
			writer.AddAttribute(HtmlTextWriterAttribute.Name, this.GroupName);
			writer.AddAttribute(HtmlTextWriterAttribute.Value, this.ValueAttribute);
			
			if (this.Checked)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Checked, "checked");
			}
			
			if (!this.Enabled)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Disabled, "disabled");
			}
			
			if (this.AutoPostBack)
			{
				if (onClick != null)
				{
					onClick += this.Page.GetPostBackClientEvent(this, String.Empty);
				}
				else
				{
					onClick = this.Page.GetPostBackClientEvent(this, String.Empty);
				}

				writer.AddAttribute(HtmlTextWriterAttribute.Onclick, onClick);
				writer.AddAttribute("language", "javascript");
			}
			else if (onClick != null)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Onclick, onClick);
			}
			
			if (this.AccessKey.Length > 0)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Accesskey, this.AccessKey);
			}
			
			if (this.TabIndex != 0)
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Tabindex, this.TabIndex.ToString(NumberFormatInfo.InvariantInfo));
			}
			
			writer.RenderBeginTag(HtmlTextWriterTag.Input);
			writer.RenderEndTag();
		}
		#endregion

		#region IPostBackDataHandler Members

		public void RaisePostDataChangedEvent()
		{
			this.OnCheckedChanged(EventArgs.Empty);
		}

		public bool LoadPostData(string postDataKey, System.Collections.Specialized.NameValueCollection postCollection)
		{
			string postData = postCollection[this.GroupName];
			bool isChecked = false;
			
			if ((postData != null) && postData.Equals(this.ValueAttribute))
			{
				if (!this.Checked)
				{
					this.Checked = true;
					isChecked = true;
				}

				return isChecked;
			}

			if (this.Checked)
			{
				this.Checked = false;
			}
			
			return isChecked;
		}

		#endregion
	}
}
